package com.ruoyi.framework.web.service;

import javax.annotation.Resource;

import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import cn.binarywang.wx.miniapp.bean.WxMaPhoneNumberInfo;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.config.WxConfig;
import com.ruoyi.common.core.domain.entity.SmUser;
import com.ruoyi.common.core.domain.model.WxLoginBody;
import com.ruoyi.common.enums.UserStatus;
import com.ruoyi.common.enums.UserType;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.http.HttpUtils;
import com.ruoyi.common.wx.AccessTokenUtil;
import com.ruoyi.ss.store.service.IStoreService;
import com.ruoyi.ss.user.service.ISmUserService;
import com.ruoyi.ss.account.service.ISmAccountService;
import com.ruoyi.ss.account.domain.SmAccountBO;
import com.ruoyi.ss.account.domain.enums.AccountType;
import com.ruoyi.system.service.*;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Component;
import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.exception.user.BlackListException;
import com.ruoyi.common.exception.user.CaptchaException;
import com.ruoyi.common.exception.user.CaptchaExpireException;
import com.ruoyi.common.exception.user.UserNotExistsException;
import com.ruoyi.common.exception.user.UserPasswordNotMatchException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.MessageUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.ip.IpUtils;
import com.ruoyi.framework.manager.AsyncManager;
import com.ruoyi.framework.manager.factory.AsyncFactory;
import com.ruoyi.framework.security.context.AuthenticationContextHolder;
import org.springframework.transaction.annotation.Transactional;

import java.util.Objects;

/**
 * 登录校验方法
 *
 * @author ruoyi
 */
@Component
public class SysLoginService
{
    @Autowired
    private TokenService tokenService;

    @Resource
    private AuthenticationManager authenticationManager;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    private ISmUserService smUserService;

    @Autowired
    private WxConfig wxConfig;

    @Autowired
    private ISmAccountService accountService;

    @Autowired
    private IStoreService smDeviceGroupService;

    /**
     * 登录验证
     *
     * @param username 用户名
     * @param password 密码
     * @param code 验证码
     * @param uuid 唯一标识
     * @return 结果
     */
    public String login(String username, String password, String code, String uuid)
    {
        // 验证码校验
        validateCaptcha(username, code, uuid);
        // 登录前置校验
        loginPreCheck(username, password);
        // 用户验证
        Authentication authentication = null;
        try
        {
            UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(username, password);
            AuthenticationContextHolder.setContext(authenticationToken);
            // 把用户登录类型放在上下文中的details属性中，在UserDetailsServiceImpl.loadUserByUsername中获取
            authenticationToken.setDetails(Constants.USER_TYPE_PC);
            // 该方法会去调用UserDetailsServiceImpl.loadUserByUsername
            authentication = authenticationManager.authenticate(authenticationToken);
        }
        catch (Exception e)
        {
            if (e instanceof BadCredentialsException)
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.password.not.match")));
                throw new UserPasswordNotMatchException();
            }
            else
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, e.getMessage()));
                throw new ServiceException(e.getMessage());
            }
        }
        finally
        {
            AuthenticationContextHolder.clearContext();
        }
        AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_SUCCESS, MessageUtils.message("user.login.success")));
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        recordLoginInfo(loginUser.getUserId());
        // 生成token
        return tokenService.createToken(loginUser);
    }

    /**
     * 校验验证码
     *
     * @param username 用户名
     * @param code 验证码
     * @param uuid 唯一标识
     * @return 结果
     */
    public void validateCaptcha(String username, String code, String uuid)
    {
        boolean captchaEnabled = configService.selectCaptchaEnabled();
        if (captchaEnabled)
        {
            String verifyKey = CacheConstants.CAPTCHA_CODE_KEY + StringUtils.nvl(uuid, "");
            String captcha = redisCache.getCacheObject(verifyKey);
            redisCache.deleteObject(verifyKey);
            if (captcha == null)
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.jcaptcha.expire")));
                throw new CaptchaExpireException();
            }
            if (!code.equalsIgnoreCase(captcha))
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.jcaptcha.error")));
                throw new CaptchaException();
            }
        }
    }

    /**
     * 登录前置校验
     * @param username 用户名
     * @param password 用户密码
     */
    public void loginPreCheck(String username, String password)
    {
        // 用户名或密码为空 错误
        if (StringUtils.isEmpty(username) || StringUtils.isEmpty(password))
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("not.null")));
            throw new UserNotExistsException();
        }
        // 密码如果不在指定范围内 错误
        if (password.length() < UserConstants.PASSWORD_MIN_LENGTH
                || password.length() > UserConstants.PASSWORD_MAX_LENGTH)
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.password.not.match")));
            throw new UserPasswordNotMatchException();
        }
        // 用户名不在指定范围内 错误
        if (username.length() < UserConstants.USERNAME_MIN_LENGTH
                || username.length() > UserConstants.USERNAME_MAX_LENGTH)
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.password.not.match")));
            throw new UserPasswordNotMatchException();
        }
        // IP黑名单校验
        String blackStr = configService.selectConfigByKey("sys.login.blackIPList");
        if (IpUtils.isMatchedIp(blackStr, IpUtils.getIpAddr()))
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("login.blocked")));
            throw new BlackListException();
        }
    }

    /**
     * 记录登录信息
     *
     * @param userId 用户ID
     */
    public void recordLoginInfo(Long userId)
    {
        SysUser sysUser = new SysUser();
        sysUser.setUserId(userId);
        sysUser.setLoginIp(IpUtils.getIpAddr());
        sysUser.setLoginDate(DateUtils.getNowDate());
        userService.updateUserProfile(sysUser);
    }

    /**
     * 前台账号密码登录
     * @param username
     * @param password
     * @return
     */
    @Deprecated
    public String appLogin(String username, String password) {
        // 登录前置校验
        loginPreCheck(username, password);
        // 用户验证
        Authentication authentication = null;
        try
        {
            UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(username, password);
            AuthenticationContextHolder.setContext(authenticationToken);
            // 把用户登录类型放在上下文中的details属性中，在UserDetailsServiceImpl.loadUserByUsername中获取
            authenticationToken.setDetails(Constants.USER_TYPE_WX);
            // 该方法会去调用UserDetailsServiceImpl.loadUserByUsername
            authentication = authenticationManager.authenticate(authenticationToken);
        }
        catch (Exception e)
        {
            if (e instanceof BadCredentialsException)
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.password.not.match")));
                throw new UserPasswordNotMatchException();
            }
            else
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, e.getMessage()));
                throw new ServiceException(e.getMessage());
            }
        }
        finally
        {
            AuthenticationContextHolder.clearContext();
        }
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        // 生成token
        return tokenService.createToken(loginUser);
    }

    /**
     * 微信登录
     */
    @Transactional
    public String wxLogin(WxLoginBody body) {

        // 通过登录授权码获取到用户信息
        WxMaJscode2SessionResult wxMaJscode2SessionResult = this.wxJsCode2Session(body.getLoginCode());

        // 通过openId查询用户，判断是否微信用户
        SmUser user = smUserService.selectSmUserByWxOpenId(wxMaJscode2SessionResult.getOpenid());

        // 不是微信老用户，则将微信与手机号绑定
        if (user == null || Objects.equals(user.getDelFlag(), UserStatus.DELETED.getCode())) {
            ServiceUtil.assertion(body.getMobileCode() == null, "请提供微信手机号授权码", 10003);

            // 通过授权码获取微信手机号
            String phoneNumber = getWxPhoneNumber(body.getMobileCode());

            // 判断用户是否注册，未注册的自动注册
            user = smUserService.selectUserByPhone(phoneNumber);
            if(ObjectUtils.isEmpty(user)){
                user = registerWx(phoneNumber, wxMaJscode2SessionResult);
            }
        }

        Authentication authentication = null;
        try {
            UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(user.getUserName(), Constants.CUSTOM_LOGIN_WX);
            // 用户名和密码等信息保存在一个上下文中，只要是同一线程等会就能拿到用户名和密码，也就是能在loadUserByUsername(String username)方法中进行密码验证等
            AuthenticationContextHolder.setContext(authenticationToken);
            // 把用户登录类型放在上下文中的details属性中，在UserDetailsServiceImpl.loadUserByUsername中获取
            authenticationToken.setDetails(Constants.USER_TYPE_WX);
            // 该方法会去调用UserDetailsServiceImpl.loadUserByUsername
            authentication = authenticationManager.authenticate(authenticationToken);
        }
        catch (Exception e) {
            if (e instanceof BadCredentialsException) {
                throw new UserPasswordNotMatchException();          //抛出账号或者密码错误的异常
            } else {
                throw new ServiceException(e.getMessage());         //抛出其他异常
            }
        } finally {
            AuthenticationContextHolder.clearContext();
        }
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        return tokenService.createToken(loginUser);
    }

    /**
     * 注册微信用户
     * @param phoneNumber 手机号
     * @param wxMaJscode2SessionResult 微信用户信息
     * @return
     */
    private SmUser registerWx(String phoneNumber, WxMaJscode2SessionResult wxMaJscode2SessionResult) {
        // 添加用户
        SmUser newUser = new SmUser();
        newUser.setUserName(phoneNumber);
        newUser.setPhonenumber(phoneNumber);
        newUser.setNickName("微信用户" + phoneNumber.substring(phoneNumber.length() - 4));
        newUser.setLoginIp(IpUtils.getIpAddr());
        newUser.setLoginDate(DateUtils.getNowDate());
        newUser.setCreateTime(DateUtils.getNowDate());
        newUser.setWxOpenId(wxMaJscode2SessionResult.getOpenid());
        smUserService.insertSmUser(newUser);

        // 添加微信账户
        SmAccountBO accountData = new SmAccountBO();
        accountData.setUserId(newUser.getUserId());
        accountData.setAccountNo(newUser.getWxOpenId());
        accountData.setAccountType(AccountType.WECHAT.getType());
        accountService.addAccount(accountData);

        return newUser;
    }

    /**
     * 使用登录码获取微信用户信息
     * @param jsCode 登录码，通过wx.login获得
     */
    private WxMaJscode2SessionResult wxJsCode2Session(String jsCode) {
        String grantType = "authorization_code";
        String url = String.format("https://api.weixin.qq.com/sns/jscode2session?appid=%s&secret=%s&js_code=%s&grant_type=%s", wxConfig.getAppId(), wxConfig.getAppSecret(), jsCode, grantType);
        String body = HttpUtils.sendGet(url);
        return JSON.parseObject(body, WxMaJscode2SessionResult.class);
    }

    /**
     * 通过授权码获取微信用户手机号
     * @param mobileCode
     * @return
     */
    private String getWxPhoneNumber(String mobileCode) {
        String url = "https://api.weixin.qq.com/wxa/business/getuserphonenumber?access_token=";

        // 根据手机号获取到用户名
        String token = AccessTokenUtil.getToken();
        url = url + token;
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("code", mobileCode);
        String post = HttpUtils.sendPost(url, jsonObject.toString());

        // 获取微信用户手机号
        JSONObject jsonObject1 = JSONObject.parseObject(post);
        String phoneInfo = jsonObject1.getString("phone_info");
        WxMaPhoneNumberInfo wxMaPhoneNumberInfo = JSONObject.parseObject(phoneInfo, WxMaPhoneNumberInfo.class);
        return wxMaPhoneNumberInfo.getPhoneNumber();
    }
}
