package com.ruoyi.ss.account.domain;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.ruoyi.common.annotation.Excel;
import com.ruoyi.common.core.domain.BaseEntity;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.common.utils.RegexpUtils;
import com.ruoyi.common.valid.EnumValid;
import com.ruoyi.common.valid.bank.BankCardInfo;
import com.ruoyi.ss.account.domain.enums.AccountType;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Pattern;

/**
 * 用户账户对象 sm_account
 *
 * @author ruoyi
 * @date 2024-03-26
 */
@Data
public class SmAccount extends BaseEntity
{
    private static final long serialVersionUID = 1L;

    /** 账户id */
    private Long accountId;

    /** 用户id */
    @Excel(name = "用户id")
    private Long userId;

    /** 账户类型：1银行卡、2微信、3支付宝 */
    @Excel(name = "账户类型：1银行卡、2微信、3支付宝")
    @EnumValid(
            clazz = AccountType.class,
            message = "非法的账户类型",
            allowNull = false,
            method = "getType",
            groups = {ValidGroup.FrontCreate.class}
    )
    private String accountType;

    /** 姓名 */
    @Excel(name = "姓名")
    private String name;

    /** 身份证 */
    @Excel(name = "身份证")
    @Pattern(regexp = RegexpUtils.IDENTITY_CARD_REGEXP, message = "身份证格式错误", groups = {ValidGroup.FrontCreate.class})
    @JsonProperty(access = JsonProperty.Access.WRITE_ONLY)
    private String idCard;

    /** 卡号（微信号/支付宝账号） */
    @Excel(name = "卡号/微信号/支付宝账号")
    @NotBlank(message = "卡号/微信号/支付宝账号不允许为空", groups = {ValidGroup.FrontCreate.class})
    private String accountNo;

    /** 手机号 */
    @Excel(name = "手机号")
    private String mobile;

    @ApiModelProperty("卡面信息")
    private BankCardInfo cardInfo;

    @ApiModelProperty("是否默认账户")
    private Boolean isDefault;

    /**
     * 数据脱敏
     */
    public void desensitization() {
        if (this.accountNo != null) {
            this.accountNo = StringUtils.substring(this.accountNo, 0, 4)
                    + "****"
                    + StringUtils.substring(this.accountNo, this.accountNo.length() - 4, this.accountNo.length());
        }
    }
}
