package com.ruoyi.ss.account.service;

import com.ruoyi.ss.account.domain.SmAccount;
import com.ruoyi.ss.account.domain.SmAccountBO;
import com.ruoyi.ss.account.domain.SmAccountQuery;
import com.ruoyi.ss.account.domain.SmAccountVo;
import com.ruoyi.ss.account.domain.enums.AccountType;

import java.util.List;
import java.util.Map;

/**
 * 用户账户Service接口
 *
 * @author ruoyi
 * @date 2024-03-26
 */
public interface ISmAccountService
{
    /**
     * 查询用户账户
     *
     * @param accountId 用户账户主键
     * @return 用户账户
     */
    public SmAccount selectSmAccountByAccountId(Long accountId);

    /**
     * 查询用户账户列表
     *
     * @param smAccount 用户账户
     * @return 用户账户集合
     */
    public List<SmAccountVo> selectSmAccountList(SmAccountQuery smAccount);

    /**
     * 新增用户账户
     *
     * @param smAccount 用户账户
     * @return 结果
     */
    public int insertSmAccount(SmAccount smAccount);

    /**
     * 修改用户账户
     *
     * @param smAccount 用户账户
     * @return 结果
     */
    public int updateSmAccount(SmAccount smAccount);

    /**
     * 批量删除用户账户
     *
     * @param accountIds 需要删除的用户账户主键集合
     * @return 结果
     */
    public int deleteSmAccountByAccountIds(Long[] accountIds);

    /**
     * 删除用户账户信息
     *
     * @param accountId 用户账户主键
     * @return 结果
     */
    public int deleteSmAccountByAccountId(Long accountId);

    /**
     * 新增账户信息
     * @param data 账户信息
     */
    boolean addAccount(SmAccountBO data);

    /**
     * 查询列表并分组
     */
    Map<String, List<SmAccountVo>> selectSmAccountGroup(SmAccountQuery dto);

    /**
     * 判断这个账号是不是用户的
     * @param userId 用户id
     * @param accountNo 账号
     */
    boolean validateUserHasAccount(Long userId, String accountNo, AccountType accountType);

    /**
     * 查询用户的微信账户
     * @param userId 用户id
     */
    SmAccountVo selectUserWxAccount(Long userId);

    /**
     * 查询用户的支付宝账户
     * @param userId 用户id
     */
    SmAccountVo selectUseAliAccount(Long userId);

    /**
     * 设置默认账户
     * @param userId 用户id
     * @param accountId 账户id
     */
    boolean setDefault(Long userId, Long accountId);
}
