package com.ruoyi.ss.businessRecord.service;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.ss.businessRecord.domain.SmBusinessRecord;
import com.ruoyi.ss.businessRecord.domain.SmBusinessRecordQuery;
import com.ruoyi.ss.businessRecord.domain.SmBusinessRecordVo;
import com.ruoyi.ss.businessRecord.mapper.SmBusinessRecordMapper;
import com.ruoyi.ss.dashboard.BillCountVo;
import com.ruoyi.ss.dashboard.BriefVo;
import com.ruoyi.ss.device.domain.SmDeviceQuery;
import com.ruoyi.ss.device.domain.enums.DeviceOnlineStatus;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.model.domain.SmModelQuery;
import com.ruoyi.ss.model.service.ModelService;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.user.domain.SmUserQuery;
import com.ruoyi.ss.user.service.ISmUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;

/**
 * 经营记录Service业务层处理
 *
 * @author ruoyi
 * @date 2024-03-19
 */
@Service
public class SmBusinessRecordServiceImpl implements ISmBusinessRecordService
{
    @Autowired
    private SmBusinessRecordMapper smBusinessRecordMapper;

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private ISmUserService userService;

    @Autowired
    private ModelService modelService;

    @Autowired
    private TransactionBillService transactionBillService;

    /**
     * 查询经营记录
     *
     * @param id 经营记录主键
     * @return 经营记录
     */
    @Override
    public SmBusinessRecordVo selectSmBusinessRecordById(Long id)
    {
        return smBusinessRecordMapper.selectSmBusinessRecordById(id);
    }

    /**
     * 查询经营记录列表
     *
     * @param smBusinessRecord 经营记录
     * @return 经营记录
     */
    @Override
    public List<SmBusinessRecordVo> selectSmBusinessRecordList(SmBusinessRecordQuery smBusinessRecord)
    {
        return smBusinessRecordMapper.selectSmBusinessRecordList(smBusinessRecord);
    }

    /**
     * 新增经营记录
     *
     * @param smBusinessRecord 经营记录
     * @return 结果
     */
    @Override
    public int insertSmBusinessRecord(SmBusinessRecord smBusinessRecord)
    {
        smBusinessRecord.setCreateTime(DateUtils.getNowDate());
        return smBusinessRecordMapper.insertSmBusinessRecord(smBusinessRecord);
    }

    /**
     * 修改经营记录
     *
     * @param smBusinessRecord 经营记录
     * @return 结果
     */
    @Override
    public int updateSmBusinessRecord(SmBusinessRecord smBusinessRecord)
    {
        return smBusinessRecordMapper.updateSmBusinessRecord(smBusinessRecord);
    }

    /**
     * 批量删除经营记录
     *
     * @param ids 需要删除的经营记录主键
     * @return 结果
     */
    @Override
    public int deleteSmBusinessRecordByIds(Long[] ids)
    {
        return smBusinessRecordMapper.deleteSmBusinessRecordByIds(ids);
    }

    /**
     * 删除经营记录信息
     *
     * @param id 经营记录主键
     * @return 结果
     */
    @Override
    public int deleteSmBusinessRecordById(Long id)
    {
        return smBusinessRecordMapper.deleteSmBusinessRecordById(id);
    }

    @Override
    public SmBusinessRecordVo selectLatestByDate(Date date) {
        SmBusinessRecordQuery dto = new SmBusinessRecordQuery();
        dto.setCreateDate(date);
        return smBusinessRecordMapper.selectLatest(dto);
    }

    @Override
    public List<SmBusinessRecordVo> selectDailyLatest(Date startDate, Date endDate) {
        SmBusinessRecordQuery dto = new SmBusinessRecordQuery();
        dto.setStartDate(startDate);
        dto.setEndDate(endDate);
        return smBusinessRecordMapper.selectDailyLatest(dto);
    }

    /**
     * 查询实时数据
     */
    @Override
    public BriefVo selectBrief() {
        BriefVo brief = new BriefVo();

        // 设备总数
        SmDeviceQuery deviceDto = new SmDeviceQuery();
        deviceDto.setDeleted(false);
        deviceDto.setIsBind(true);
        brief.setDeviceCount(deviceService.selectCount(deviceDto));

        // 在线设备总数
        SmDeviceQuery onlineDto = new SmDeviceQuery();
        onlineDto.setOnlineStatus(DeviceOnlineStatus.ONLINE.getStatus());
        onlineDto.setDeleted(false);
        onlineDto.setIsBind(true);
        brief.setOnlineCount(deviceService.selectCount(onlineDto));

        // 型号总数
        SmModelQuery modelDto = new SmModelQuery();
        modelDto.setDeleted(false);
        brief.setModelCount(modelService.selectCount(modelDto));

        // 商户总数
        SmUserQuery userDto = new SmUserQuery();
        userDto.setIsMch(true);
        brief.setUserCount(userService.selectCount(userDto));

        // 租户总数
        SmUserQuery tenantDto = new SmUserQuery();
        tenantDto.setIsMch(false);
        brief.setTenantCount(userService.selectCount(tenantDto));

        // 充值金额总数
        TransactionBillQuery rechargeDto = new TransactionBillQuery();
        rechargeDto.setStatus(TransactionBillStatus.SUCCESS.getStatus());
        List<BillCountVo> billCount = transactionBillService.selectCount(rechargeDto);
        if (!CollectionUtils.isEmpty(billCount)) {
            brief.setRechargeAmount(billCount.get(0).getRecharge());
        }

        return brief;
    }

    /**
     * 记录数据
     */
    @Override
    public void record() {
        BriefVo brief = this.selectBrief();

        SmBusinessRecord data = new SmBusinessRecord();
        data.setTotalDevice(brief.getDeviceCount());
        data.setTotalOnline(brief.getOnlineCount());
        data.setTotalModel(brief.getModelCount());
        data.setTotalUser(brief.getUserCount() + brief.getTenantCount());
        data.setTotalRecharge(brief.getRechargeAmount());
        this.insertSmBusinessRecord(data);
    }
}
