package com.ruoyi.ss.channel.service;

import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.ss.channel.domain.SmChannel;
import com.ruoyi.ss.channel.domain.SmChannelQuery;
import com.ruoyi.ss.channel.mapper.SmChannelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 充值渠道Service业务层处理
 *
 * @author ruoyi
 * @date 2024-04-15
 */
@Service
public class SmChannelServiceImpl implements ISmChannelService
{
    @Autowired
    private SmChannelMapper smChannelMapper;

    /**
     * 查询充值渠道
     *
     * @param channelId 充值渠道主键
     * @return 充值渠道
     */
    @Override
    public SmChannel selectSmChannelByChannelId(Long channelId)
    {
        return smChannelMapper.selectSmChannelByChannelId(channelId);
    }

    /**
     * 查询充值渠道列表
     *
     * @param smChannel 充值渠道
     * @return 充值渠道
     */
    @Override
    public List<SmChannel> selectSmChannelList(SmChannelQuery smChannel)
    {
        return smChannelMapper.selectSmChannelList(smChannel);
    }

    /**
     * 新增充值渠道
     *
     * @param smChannel 充值渠道
     * @return 结果
     */
    @Override
    public int insertSmChannel(SmChannel smChannel)
    {
        return smChannelMapper.insertSmChannel(smChannel);
    }

    /**
     * 修改充值渠道
     *
     * @param smChannel 充值渠道
     * @return 结果
     */
    @Override
    public int updateSmChannel(SmChannel smChannel)
    {
        return smChannelMapper.updateSmChannel(smChannel);
    }

    /**
     * 批量删除充值渠道
     *
     * @param channelIds 需要删除的充值渠道主键
     * @return 结果
     */
    @Override
    public int deleteSmChannelByChannelIds(Long[] channelIds)
    {
        return smChannelMapper.deleteSmChannelByChannelIds(channelIds);
    }

    /**
     * 删除充值渠道信息
     *
     * @param channelId 充值渠道主键
     * @return 结果
     */
    @Override
    public int deleteSmChannelByChannelId(Long channelId)
    {
        return smChannelMapper.deleteSmChannelByChannelId(channelId);
    }

    @Override
    public List<SmChannel> selectEnabledRechargeList() {
        SmChannelQuery dto = new SmChannelQuery();
        dto.setEnabled(true);
        return this.selectSmChannelList(dto);
    }

    /**
     * 查询充值渠道映射表
     *
     * @param query     查询条件
     * @param keyMapper 映射函数
     */
    @Override
    public <K> Map<K, SmChannel> selectMap(SmChannelQuery query, Function<? super SmChannel, ? extends K> keyMapper) {
        List<SmChannel> list = this.selectSmChannelList(query);
        if (CollectionUtils.isEmptyElement(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(keyMapper, item -> item));
    }
}
