package com.ruoyi.ss.deviceTenant.service;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.ss.deviceTenant.domain.SmDeviceTenant;
import com.ruoyi.ss.deviceTenant.domain.SmDeviceTenantQuery;
import com.ruoyi.ss.deviceTenant.mapper.SmDeviceTenantMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 设备与租户绑定（中间）Service业务层处理
 *
 * @author ruoyi
 * @date 2024-03-15
 */
@Service
public class SmDeviceTenantServiceImpl implements ISmDeviceTenantService
{
    @Autowired
    private SmDeviceTenantMapper smDeviceTenantMapper;

    /**
     * 查询设备与租户绑定（中间）
     *
     * @param id 设备与租户绑定（中间）主键
     * @return 设备与租户绑定（中间）
     */
    @Override
    public SmDeviceTenant selectSmDeviceTenantById(Long id)
    {
        return smDeviceTenantMapper.selectSmDeviceTenantById(id);
    }

    /**
     * 查询设备与租户绑定（中间）列表
     *
     * @param smDeviceTenant 设备与租户绑定（中间）
     * @return 设备与租户绑定（中间）
     */
    @Override
    public List<SmDeviceTenant> selectSmDeviceTenantList(SmDeviceTenantQuery smDeviceTenant)
    {
        return smDeviceTenantMapper.selectSmDeviceTenantList(smDeviceTenant);
    }

    /**
     * 新增设备与租户绑定（中间）
     *
     * @param smDeviceTenant 设备与租户绑定（中间）
     * @return 结果
     */
    @Override
    public int insertSmDeviceTenant(SmDeviceTenant smDeviceTenant)
    {
        smDeviceTenant.setCreateTime(DateUtils.getNowDate());
        return smDeviceTenantMapper.insertSmDeviceTenant(smDeviceTenant);
    }

    /**
     * 修改设备与租户绑定（中间）
     *
     * @param smDeviceTenant 设备与租户绑定（中间）
     * @return 结果
     */
    @Override
    public int updateSmDeviceTenant(SmDeviceTenant smDeviceTenant)
    {
        return smDeviceTenantMapper.updateSmDeviceTenant(smDeviceTenant);
    }

    /**
     * 批量删除设备与租户绑定（中间）
     *
     * @param ids 需要删除的设备与租户绑定（中间）主键
     * @return 结果
     */
    @Override
    public int deleteSmDeviceTenantByIds(Long[] ids)
    {
        return smDeviceTenantMapper.deleteSmDeviceTenantByIds(ids);
    }

    /**
     * 删除设备与租户绑定（中间）信息
     *
     * @param id 设备与租户绑定（中间）主键
     * @return 结果
     */
    @Override
    public int deleteSmDeviceTenantById(Long id)
    {
        return smDeviceTenantMapper.deleteSmDeviceTenantById(id);
    }

    @Override
    public boolean selectIsBind(Long userId, Long deviceId) {
        SmDeviceTenantQuery dto = new SmDeviceTenantQuery();
        dto.setDeviceId(deviceId);
        dto.setTenantId(userId);
        return !CollectionUtils.isEmpty(smDeviceTenantMapper.selectSmDeviceTenantList(dto));
    }

    @Override
    public int unBind(Long tenantId, Long deviceId) {
        return smDeviceTenantMapper.unBind(tenantId, deviceId);
    }

    @Override
    public Map<Long, List<SmDeviceTenant>> selectGroupByDeviceIds(List<Long> deviceIds) {
        if (CollectionUtils.isEmpty(deviceIds)) {
            return Collections.emptyMap();
        }
        SmDeviceTenantQuery dto = new SmDeviceTenantQuery();
        dto.setDeviceIds(deviceIds);
        List<SmDeviceTenant> list = this.selectSmDeviceTenantList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.groupingBy(SmDeviceTenant::getDeviceId));
    }

    @Override
    public List<SmDeviceTenant> selectByDeviceId(Long deviceId) {
        if (deviceId == null) {
            return Collections.emptyList();
        }
        SmDeviceTenantQuery dto = new SmDeviceTenantQuery();
        dto.setDeviceId(deviceId);
        return this.selectSmDeviceTenantList(dto);
    }

}
