package com.ruoyi.ss.store.service;

import com.ruoyi.common.core.domain.ValidateResult;
import com.ruoyi.ss.store.domain.Store;

import java.util.List;

/**
 * 2024/4/27
 *
 * @author wjh
 */
public interface StoreValidator {

    /**
     * 逻辑删除前校验
     * @param ids 店铺id列表
     */
    ValidateResult preLogicDel(List<Long> ids);

    /**
     * 前台逻辑删除前校验
     * @param storeIds 店铺id列表
     */
    ValidateResult preLogicDelByApp(List<Long> storeIds);

    /**
     * 更新前校验
     * @param data 数据
     */
    ValidateResult preUpdate(Store data);

    /**
     * 创建前校验
     * @param data 数据
     */
    ValidateResult preCreate(Store data);

    /**
     * 前台创建前校验
     * @param data 数据
     */
    ValidateResult preCreateByApp(Store data);

    /**
     * 前台更新前校验
     * @param data
     * @return
     */
    ValidateResult preUpdateByApp(Store data);

    /**
     * 排序前校验
     * @param list 排序列表
     */
    ValidateResult preChangeSortByApp(List<Store> list);

    /**
     * 判断店铺是否存在设备
     * @param ids 店铺id列表
     */
    boolean hasDevice(List<Long> ids);

    /**
     * 判断店铺是否属于用户
     */
    boolean isStoreBelongUser(List<Long> storeId, Long userId);

    /**
     * 判断店铺是否存在
     * @param storeIds
     */
    boolean isExist(List<Long> storeIds);

    /**
     * 设置默认店铺前校验
     */
    ValidateResult preSetDefaultByApp(Long userId, Long storeId);
}
