package com.ruoyi.ss.user.service.impl;

import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.ss.device.domain.SmDeviceCountVO;
import com.ruoyi.ss.device.domain.SmDeviceQuery;
import com.ruoyi.ss.device.domain.enums.DeviceGroupBy;
import com.ruoyi.ss.device.domain.enums.DeviceGroupByTable;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.store.service.IStoreService;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.ss.user.service.UserAssembler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wjh
 * 2024/5/6
 */
@Service
public class UserAssemblerImpl implements UserAssembler {

    @Autowired
    private IStoreService storeService;

    @Autowired
    private DeviceService deviceService;

    /**
     * 拼接用户店铺数量
     *
     * @param list
     */
    @Override
    public void assembleStoreCount(List<SmUserVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }

        Map<Long, Integer> map = storeService.selectCountMapByUserIds(list.stream().map(SmUserVo::getUserId).collect(Collectors.toList()));

        for (SmUserVo user : list) {
            Integer count = map.get(user.getUserId());
            user.setStoreCount(count == null ? 0 : count);
        }
    }

    /**
     * 拼接用户设备数量
     *
     * @param list
     */
    @Override
    public void assembleDeviceCount(List<SmUserVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }

        SmDeviceQuery query = new SmDeviceQuery();
        query.setUserIds(list.stream().map(SmUserVo::getUserId).collect(Collectors.toList()));
        query.setGroupBy(DeviceGroupBy.user_id.name());
        query.setGroupByTable(DeviceGroupByTable.su.name());
        Map<Long, Integer> map = deviceService.selectCommonCountMap(query, SmDeviceCountVO::getUserId);

        for (SmUserVo user : list) {
            Integer count = map.get(user.getUserId());
            user.setDeviceCount(count == null ? 0 : count);
        }
    }
}
